/*
    AutoX11, a tool to automate tasks in an X11 environment
    Copyright (C) 2014  Alicia <alicia@ion.nu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include <gtksourceview/gtksource.h>
#include <gdk/gdkx.h>
#include "grab.h"
#include "autox11.h"
#include "designer.h"

GtkWidget* srcview;
GtkWidget* mainwindow;
const char* currentfile=0;

char tooltip(GtkWidget* widget, int x, int y, char kbdmode, GtkTooltip* tooltip, void* data)
{
  GtkTextIter start;
  gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(widget), &start, x, y);
  GtkTextIter end=start;
  // Seek to whitespace, delimiting arguments/filenames
  while(!strchr(" \r\n\t", (char)gtk_text_iter_get_char(&start)) && !gtk_text_iter_is_start(&start)){gtk_text_iter_backward_char(&start);}
  while(!strchr(" \r\n\t", (char)gtk_text_iter_get_char(&end)) && !gtk_text_iter_is_end(&end)){gtk_text_iter_forward_char(&end);}
  if(strchr(" \r\n\t", (char)gtk_text_iter_get_char(&start))){gtk_text_iter_forward_char(&start);}
  GtkTextBuffer* buffer=gtk_text_iter_get_buffer(&start);
  // Get the argument
  char* filename=gtk_text_buffer_get_text(buffer, &start, &end, 0);
  // Try to load the image, return 0 if it fails (i.e. it wasn't a filename or didn't lead to an image)
  GdkPixbuf* img=gdk_pixbuf_new_from_file(filename, 0);
  free(filename);
  if(!img){return 0;}
  // Set tooltip
  gtk_tooltip_set_icon(tooltip, img);
  g_object_unref(img);
  return 1;
}

void openfile2(const char* filename)
{
  struct stat st;
  stat(filename, &st);
  char* buf=malloc(st.st_size);
  int f=open(filename, O_RDONLY);
  read(f, buf, st.st_size); // TODO: do we need to worry about incomplete reads when reading from a filesystem?
  close(f);
  GtkTextBuffer* buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(srcview));
  gtk_text_buffer_set_text(buffer, buf, st.st_size);
  free(buf);
  gtk_text_buffer_set_modified(buffer, 0);
  free((void*)currentfile);
  currentfile=strdup(filename);
// TODO: chdir to wherever the script is? this would ensure that images are found for tooltips, and that new images are saved to the right place
// TODO: where should images for not yet saved scripts be stored? can we keep a list to move them once the user saves?
// or maybe changing directory is the wrong way to go, considering that your working directory doesn't change when you call a script, unless the script does so
}

void openfile(GtkMenuItem* menuitem, void* data)
{
  // Check if the currently open script has unsaved changes
  GtkTextBuffer* buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(srcview));
  if(gtk_text_buffer_get_modified(buffer))
  {
    GtkWidget* q=gtk_dialog_new_with_buttons("Unsaved changes", GTK_WINDOW(mainwindow), GTK_DIALOG_MODAL, "_Yes", GTK_RESPONSE_YES, "_No", GTK_RESPONSE_NO, (char*)0);
    GtkWidget* content=gtk_dialog_get_content_area(GTK_DIALOG(q));
    gtk_container_add(GTK_CONTAINER(content), gtk_label_new("The current script has unsaved changes, discard it and open another script anyway?"));
    gtk_widget_show_all(q);
    if(gtk_dialog_run(GTK_DIALOG(q))!=GTK_RESPONSE_YES)
    {
      gtk_widget_destroy(q);
      return;
    }
    gtk_widget_destroy(q);
  }

  GtkWidget* dialog=gtk_file_chooser_dialog_new("Open script", GTK_WINDOW(mainwindow), GTK_FILE_CHOOSER_ACTION_OPEN, "_Open", GTK_RESPONSE_ACCEPT, "_Cancel", GTK_RESPONSE_CANCEL, (char*)0);
  int res=gtk_dialog_run(GTK_DIALOG(dialog));
  if(res==GTK_RESPONSE_ACCEPT)
  {
    char* filename=gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
//    printf("Loading '%s'\n", filename);
    openfile2(filename);
  }
  gtk_widget_destroy(dialog);
}

void savefile2(const char* filename)
{
// printf("Saving to '%s'\n", filename);
  int f=open(filename, O_WRONLY|O_TRUNC|O_CREAT, 0755);
  GtkTextBuffer* buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(srcview));
  GtkTextIter start;
  GtkTextIter end;
  memset(&start, 0, sizeof(start));
  memset(&end, 0, sizeof(end));
  gtk_text_buffer_get_start_iter(buffer, &start);
  gtk_text_buffer_get_end_iter(buffer, &end);
  char* buf=gtk_text_buffer_get_text(buffer, &start, &end, 0);
  write(f, buf, strlen(buf));
  close(f);
  free(buf);
  gtk_text_buffer_set_modified(buffer, 0);
  free((void*)currentfile);
  currentfile=strdup(filename);
}

void savefile(GtkMenuItem* menuitem, void* saveas)
{
  if(!saveas && currentfile){savefile2(currentfile); return;}
  // Set up save dialog
  GtkWidget* dialog=gtk_file_chooser_dialog_new("Save script", GTK_WINDOW(mainwindow), GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", GTK_RESPONSE_ACCEPT, "_Cancel", GTK_RESPONSE_CANCEL, (char*)0);
  gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), 1);
  if(currentfile)
  {
    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), currentfile);
  }else{
    gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), "new_script.sh");
  }
  GtkFileFilter* filter=gtk_file_filter_new();
  gtk_file_filter_set_name(filter, "Shell script (*.sh)");
  gtk_file_filter_add_pattern(filter, "*.sh");
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
  filter=gtk_file_filter_new();
  gtk_file_filter_set_name(filter, "Any filetype");
  gtk_file_filter_add_pattern(filter, "*");
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
  // Handle the response
  int res=gtk_dialog_run(GTK_DIALOG(dialog));
  if(res==GTK_RESPONSE_ACCEPT)
  {
    char* filename=gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
    savefile2(filename);
  }
  gtk_widget_destroy(dialog);
}

int main(int argc, char** argv)
{
  const char* loadfile=0;
  int i;
  for(i=1; i<argc; ++i)
  {
    // TODO: better argument handling, at least a --help/-h option?
    loadfile=argv[i];
  }

  gtk_init(&argc, &argv);
  mainwindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(mainwindow), "AutoX11 designer");
  gtk_window_set_default_size(GTK_WINDOW(mainwindow), 300, 400);
  void quit(GtkWidget* obj, void* d)
  {
    gtk_main_quit();
  }
  g_signal_connect(mainwindow, "destroy", G_CALLBACK(quit), 0);
  char asksave(GtkWidget* widget, GdkEvent* event, void* data)
  {
    // Check if the open script has unsaved changes
    GtkTextBuffer* buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(srcview));
    if(!gtk_text_buffer_get_modified(buffer)){return 0;}
    GtkWidget* q=gtk_dialog_new_with_buttons("Unsaved changes", GTK_WINDOW(mainwindow), GTK_DIALOG_MODAL, "_Yes", GTK_RESPONSE_YES, "_No", GTK_RESPONSE_NO, (char*)0);
    GtkWidget* content=gtk_dialog_get_content_area(GTK_DIALOG(q));
    gtk_container_add(GTK_CONTAINER(content), gtk_label_new("The current script has unsaved changes, quit anyway?"));
    gtk_widget_show_all(q);
    if(gtk_dialog_run(GTK_DIALOG(q))!=GTK_RESPONSE_YES)
    {
      gtk_widget_destroy(q);
      return 1;
    }
    gtk_widget_destroy(q);
    return 0;
  }
  g_signal_connect(mainwindow, "delete-event", G_CALLBACK(asksave), 0);
  GtkWidget* vbox=gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);

  gtk_container_add(GTK_CONTAINER(mainwindow), vbox);
  // Menu bar
  GtkWidget* menubar=gtk_menu_bar_new();
  gtk_box_pack_start(GTK_BOX(vbox), menubar, 0, 0, 0);
  // File menu
  GtkWidget* item=gtk_menu_item_new_with_mnemonic("_File");
  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
  GtkWidget* menu=gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
  // Open
  item=gtk_menu_item_new_with_mnemonic("_Open");
  g_signal_connect(item, "activate", G_CALLBACK(openfile), 0);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  // TODO: make these mnemonics work as hotkeys when the sourceview is selected
  // Save
  item=gtk_menu_item_new_with_mnemonic("_Save");
  g_signal_connect(item, "activate", G_CALLBACK(savefile), 0);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  // Save as
  item=gtk_menu_item_new_with_mnemonic("Save _as");
  g_signal_connect(item, "activate", G_CALLBACK(savefile), item);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  // TODO: A "run script" option? should be easy to just pass it to sh -c '<script>', but it would be nicer if we can highlight commands as they happen. Maybe sprinkle the script with 'echo <identifier> >&3' between the commands

  GtkWidget* hbox=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, 0, 0, 0);
  // Action buttons
  GtkWidget* actionlabel=gtk_label_new("Add action:");
  gtk_box_pack_start(GTK_BOX(hbox), actionlabel, 0, 0, 0);
  // "Click on"
  GtkWidget* button=gtk_button_new_with_label("Click on");
  gtk_box_pack_start(GTK_BOX(hbox), button, 0, 0, 0);
  g_signal_connect(button, "clicked", G_CALLBACK(add_imgcmd), IMGCMD_CLICK);
  // "Wait for"
  button=gtk_button_new_with_label("Wait for");
  gtk_box_pack_start(GTK_BOX(hbox), button, 0, 0, 0);
  g_signal_connect(button, "clicked", G_CALLBACK(add_imgcmd), IMGCMD_WAIT);
  // "Type"
  button=gtk_button_new_with_label("Type");
  gtk_box_pack_start(GTK_BOX(hbox), button, 0, 0, 0);
  g_signal_connect(button, "clicked", G_CALLBACK(add_typecmd), 0);
  // "Key"
  button=gtk_button_new_with_label("Key");
  gtk_box_pack_start(GTK_BOX(hbox), button, 0, 0, 0);
  g_signal_connect(button, "clicked", G_CALLBACK(add_keycmd), 0);

  GtkWidget* scroll=gtk_scrolled_window_new(0, 0);
  gtk_box_pack_start(GTK_BOX(vbox), scroll, 1, 1, 0);
  // Pick 'sh' as the language
  GtkSourceLanguageManager* lm=gtk_source_language_manager_get_default();
  GtkSourceLanguage* lang=gtk_source_language_manager_get_language(lm, "sh");
  GtkSourceBuffer* buffer=gtk_source_buffer_new_with_language(lang);
  gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), "#!/bin/sh\n", 10);
  gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(buffer), 0);
  srcview=gtk_source_view_new_with_buffer(buffer);
  PangoFontDescription* font=pango_font_description_new();
  // TODO: loop through all available fonts and list monospace ones? or just expand with more known monospace fonts
  pango_font_description_set_family(font, "monospace,lucida console,droid sans mono,dejavu sans mono,consolas,andale mono,courier");
  gtk_widget_override_font(srcview, font);
  gtk_widget_set_has_tooltip(srcview, 1);
  g_signal_connect(srcview, "query-tooltip", G_CALLBACK(tooltip), 0);

  gtk_container_add(GTK_CONTAINER(scroll), srcview);
  gtk_widget_show_all(mainwindow);
  if(loadfile){openfile2(loadfile);}
  gtk_main();
  return 0;
}
