/*
    AutoX11, a tool to automate tasks in an X11 environment
    Copyright (C) 2014  Alicia <alicia@ion.nu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>
#include <gtk/gtk.h>
#include "autox11.h"
#include "designer.h"

void add_keycmd(GtkButton* unused, void* d)
{
  GtkWidget* dialog=gtk_dialog_new_with_buttons("AutoX11 designer, key", GTK_WINDOW(mainwindow), GTK_DIALOG_MODAL, "_OK", GTK_RESPONSE_ACCEPT, "_Cancel", GTK_RESPONSE_CANCEL, (char*)0);
  GtkWidget* content=gtk_dialog_get_content_area(GTK_DIALOG(dialog));
  GtkWidget* vbox=gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
  gtk_container_add(GTK_CONTAINER(content), vbox);
  GtkWidget* keyselect=gtk_combo_box_text_new_with_entry();
  // List all keys
  int i;
  for(i=0; keylist[i]; ++i)
  {
    gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(keyselect), 0, keylist[i]);
  }
  gtk_box_pack_start(GTK_BOX(vbox), keyselect, 0, 0, 0);
  // Press (-p)
  GtkWidget* opt_press=gtk_check_button_new_with_label("Only press the key");
  gtk_box_pack_start(GTK_BOX(vbox), opt_press, 0, 0, 0);
  // Release (-r)
  GtkWidget* opt_release=gtk_check_button_new_with_label("Only release the key");
  gtk_box_pack_start(GTK_BOX(vbox), opt_release, 0, 0, 0);
  // Make the options mutually exclusive
  void onlyone(GtkToggleButton* this, GtkToggleButton* other)
  {
    if(gtk_toggle_button_get_active(this))
    {
      gtk_toggle_button_set_active(other, 0);
    }
  }
  g_signal_connect(opt_press, "toggled", G_CALLBACK(onlyone), opt_release);
  g_signal_connect(opt_release, "toggled", G_CALLBACK(onlyone), opt_press);
  gtk_widget_show_all(dialog);
  if(gtk_dialog_run(GTK_DIALOG(dialog))==GTK_RESPONSE_ACCEPT)
  {
    GtkTextBuffer* buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(srcview));
    GValue cursor=G_VALUE_INIT;
    g_value_init(&cursor, G_TYPE_INT);
    g_object_get_property(G_OBJECT(buffer), "cursor-position", &cursor);
    GtkTextIter iter;
    gtk_text_buffer_get_iter_at_offset(buffer, &iter, g_value_get_int(&cursor));
    char within=!(gtk_text_iter_starts_line(&iter)&&gtk_text_iter_ends_line(&iter));
    gtk_text_buffer_insert_at_cursor(buffer, "autox11 key ", 12);
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opt_press)))
    {
      gtk_text_buffer_insert_at_cursor(buffer, "-p ", 3);
    }
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opt_release)))
    {
      gtk_text_buffer_insert_at_cursor(buffer, "-r ", 3);
    }
    const char* text=gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(keyselect));
    gtk_text_buffer_insert_at_cursor(buffer, text, strlen(text));
    // Don't add a newline if the command was inserted into an existing expression
    if(!within)
    {
      gtk_text_buffer_insert_at_cursor(buffer, "\n", 1);
    }
  }
  gtk_widget_destroy(dialog);
}
